@@ -82,22 +82,19 @@ sub as_ipv4 {
 
 sub as_ipv6 { shift->_as_ipv6 }
 
-sub mask { Net::Silk::IPAddr::_mask($_[0], hex_cast($_[1])) }
+sub mask { $_[0]->_xs_mask($_[1]) }
 
 sub _clone { $_[0]->new($_[0]) }
 
-sub _cmp { Net::Silk::IPAddr::_xscmp($_[0], hex_cast($_[1])) }
+sub _cmp { $_[0]->_xs_cmp($_[1]) }
 
-sub _gt { Net::Silk::IPAddr::_xscmp($_[0], hex_cast($_[1])) >  0 }
-sub _ge { Net::Silk::IPAddr::_xscmp($_[0], hex_cast($_[1])) >= 0 }
-sub _lt { Net::Silk::IPAddr::_xscmp($_[0], hex_cast($_[1])) <  0 }
-sub _le { Net::Silk::IPAddr::_xscmp($_[0], hex_cast($_[1])) <= 0 }
+sub _gt { $_[0]->_cmp($_[1]) >  0 }
+sub _ge { $_[0]->_cmp($_[1]) >= 0 }
+sub _lt { $_[0]->_cmp($_[1]) <  0 }
+sub _le { $_[0]->_cmp($_[1]) <= 0 }
 
-sub _eq  { defined $_[1] &&
-           Net::Silk::IPAddr::_xscmp($_[0], hex_cast($_[1])) == 0 }
-
-sub _ne  { (! defined $_[1]) ||
-           Net::Silk::IPAddr::_xscmp($_[0], hex_cast($_[1])) != 0 }
+sub _eq  {    defined $_[1]  && $_[0]->_cmp($_[1]) == 0 }
+sub _ne  { (! defined $_[1]) || $_[0]->_cmp($_[1]) != 0 }
 
 ###
 
@@ -112,6 +109,21 @@ use Net::Silk qw( :basic );
 
 use constant is_ipv6 => 0;
 
+sub new { $_[0]->_new($_[1]) }
+
+sub _cmp {
+  my $self = $_[0];
+  my $other = $_[1];
+  my $res;
+  eval { $res = $self->_xs_cmp($other) };
+  if ($@) {
+    # possibly a ipv6 out of ipv4 range...but don't parse numbers
+    # in that case
+    $res = (SILK_IPADDR_CLASS->new($other)->_cmp($self)) * -1;
+  }
+  $res;
+}
+
 ###
 
 package Net::Silk::IPv6Addr;
@@ -124,41 +136,11 @@ use base qw( Net::Silk::IPAddr );
 use Net::Silk qw( :basic );
 
 use Math::Int128 qw( net_to_uint128 );
+use Math::Int128::die_on_overflow;
 
 use constant is_ipv6 => 1;
 
-use overload (
-  '0+'  => \&num,
-  '<=>' => \&_cmp,
-  '>'   => \&_gt,
-  '<'   => \&_lt,
-  '>='  => \&_ge,
-  '<='  => \&_le,
-  '=='  => \&_eq,
-  '!='  => \&_ne,
-  'cmp' => \&_cmp,
-  'gt'  => \&_gt,
-  'lt'  => \&_lt,
-  'ge'  => \&_ge,
-  'le'  => \&_le,
-  'eq'  => \&_eq,
-  'ne'  => \&_ne,
-  '='   => \&_clone,
-);
-
-sub _cmp { Net::Silk::IPv6Addr::_xscmp($_[0], hex_cast($_[1])) }
-
-sub _gt { Net::Silk::IPv6Addr::_xscmp($_[0], hex_cast($_[1])) >  0 }
-sub _ge { Net::Silk::IPv6Addr::_xscmp($_[0], hex_cast($_[1])) >= 0 }
-sub _lt { Net::Silk::IPv6Addr::_xscmp($_[0], hex_cast($_[1])) <  0 }
-sub _le { Net::Silk::IPv6Addr::_xscmp($_[0], hex_cast($_[1])) <= 0 }
-
-sub _eq  {
-  defined $_[1] &&
-           Net::Silk::IPv6Addr::_xscmp($_[0], hex_cast($_[1])) == 0 }
-
-sub _ne  { (! defined $_[1]) ||
-           Net::Silk::IPv6Addr::_xscmp($_[0], hex_cast($_[1])) != 0 }
+sub new { $_[0]->_new($_[1]) }
 
 sub num { net_to_uint128(Net::Silk::IPAddr::num($_[0])) }
 
