@@ -297,6 +297,23 @@ Net::Silk::Pmap - SiLK Prefix Map interface
 =head1 SYNOPSIS
 
   use Net::Silk::Pmap;
+  use Net::Silk::IPSet;
+
+  my $addys  = Net::Silk::IPSet->load("netblocks.set");
+  my $labels = Net::Silk::Pmap->load("services.pmap");
+
+  while (my $ip = <$addys>) {
+    my $svc = $labels{$ip}; # or $labels->get($ip)
+    print "$ip $svc" unless $svc eq 'unknown';
+  }
+
+  while (my $block = <$labels>) {
+    my($lo, $hi, $label) = @$block;
+    my $block = Net::Silk::IPSet->new("$lo-$hi");
+    if ($block && $addys) {
+      print "$lo-$hi $label\n";
+    }
+  }
 
 =head1 DESCRIPTION
 
@@ -368,7 +385,7 @@ L<Net::Silk>, L<Net::Silk::RWRec>, L<Net::Silk::IPSet>, L<Net::Silk::IPWildcard>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (C) 2011-2014 by Carnegie Mellon University
+Copyright (C) 2011-2015 by Carnegie Mellon University
 
 Use of the Net-Silk library and related source code is subject to the
 terms of the following licenses:
